@@ -42,7 +42,6 @@
 //use the 'major' parameter to override the default major number (83)
 /* mknod /dev/mga_vid c 83 0 */
 
-#include <linux/config.h>
 #include <linux/version.h>
 #include <linux/module.h>
 #include <linux/moduleparam.h>
@@ -270,7 +269,7 @@ typedef struct mga_card_s {
 	size_t param_buff_len;
 	bes_registers_t regs;
 	uint32_t vid_in_use;
-	uint32_t is_g400;
+	uint32_t card_type;
 	uint32_t vid_src_ready;
 	uint32_t vid_overlay_on;
 
@@ -459,7 +458,7 @@ if(!restore){
 
 	// Backend Scaler
 	writel( card->regs.besctl,      card->mmio_base + BESCTL); 
-	if(card->is_g400)
+	if(card->card_type != MGA_G200)
 		writel( card->regs.beslumactl,  card->mmio_base + BESLUMACTL); 
 	writel( card->regs.bespitch,    card->mmio_base + BESPITCH); 
 
@@ -471,7 +470,7 @@ if(!restore){
 	writel( card->regs.besb1corg,   card->mmio_base + BESB1CORG);
 	writel( card->regs.besb2org,    card->mmio_base + BESB2ORG);
 	writel( card->regs.besb2corg,   card->mmio_base + BESB2CORG);
-	if(card->is_g400) 
+	if(card->card_type != MGA_G200) 
 	{
 		writel( card->regs.besa1c3org,  card->mmio_base + BESA1C3ORG);
 		writel( card->regs.besa2c3org,  card->mmio_base + BESA2C3ORG);
@@ -551,7 +550,7 @@ switch(config->format){
                     + (1<<17)   // 4:2:0 mode
                     + (1<<18);  // dither enabled
 #if 0
-	if(card->is_g400)
+	if(card->card_type != MGA_G200)
 	{
 		//zoom disabled, zoom filter disabled, 420 3 plane format, proc amp
 		//disabled, rgb mode disabled 
@@ -595,6 +594,11 @@ switch(config->format){
 	return -1;
 }
 
+	// disable BES vertical filter if width is greater than 1024
+	// and if we have a G550
+	if(sw > 1024 && card->card_type == MGA_G550)
+		 card->regs.besctl &= ~(1<<11);
+
 	// setting black&white mode 
 	card->regs.besctl|=(card->regs.blackie<<20); 
 
@@ -640,12 +644,12 @@ if(config->format==MGA_VID_FORMAT_YV12
  ||config->format==MGA_VID_FORMAT_I420
  ){
         // planar YUV frames:
-	if (card->is_g400) 
+	if (card->card_type != MGA_G200) 
 		baseadrofs = ( ( (ofstop * card->regs.besviscal ) / 4 ) >> 16 ) * card->regs.bespitch;
 	else 
 		baseadrofs = ( ( ( ofstop * card->regs.besviscal ) / 2 ) >> 16 ) * card->regs.bespitch;
 
-    if(config->format==MGA_VID_FORMAT_YV12 || !card->is_g400){
+    if(config->format==MGA_VID_FORMAT_YV12 || (card->card_type == MGA_G200)){
 	card->regs.besa1corg = (uint32_t) card->src_base + baseadrofs + card->regs.bespitch * sh ;
 	card->regs.besa2corg = (uint32_t) card->src_base + baseadrofs + 1*frame_size + card->regs.bespitch * sh;
 	card->regs.besb1corg = (uint32_t) card->src_base + baseadrofs + 2*frame_size + card->regs.bespitch * sh;
@@ -700,12 +704,22 @@ static int mga_vid_ioctl(struct inode *inode, struct file *file, unsigned int cm
 				printk(KERN_ERR "mga_vid: failed copy from userspace\n");
 				return(-EFAULT);
 			}
+
+			// return some information back to user space in case we hit an error
+			card->config.card_type = card->card_type;
+			card->config.ram_size = card->ram_size;
+			if (copy_to_user((mga_vid_config_t *) arg, &card->config, sizeof(mga_vid_config_t)))
+			{
+				printk(KERN_ERR "mga_vid: failed copy to userspace\n");
+				return(-EFAULT);
+			}
+
 			if(card->config.version != MGA_VID_VERSION){
 				printk(KERN_ERR "mga_vid: incompatible version! driver: %X  requested: %X\n",MGA_VID_VERSION,card->config.version);
 				return(-EFAULT);
 			}
 
-			if(card->config.frame_size==0 || card->config.frame_size>1024*768*2){
+			if(card->config.frame_size==0 || card->config.frame_size>1920*1080*2){
 				printk(KERN_ERR "mga_vid: illegal frame_size: %d\n",card->config.frame_size);
 				return(-EFAULT);
 			}
@@ -715,8 +729,10 @@ static int mga_vid_ioctl(struct inode *inode, struct file *file, unsigned int cm
 				return(-EFAULT);
 			}
 			
-			if(card->config.src_width>1023 || card->config.src_height>1023){
-				printk(KERN_ERR "mga_vid: overlay sizes bigger than 1023x1023 are not supported, rescale the picture in software\n");
+			if( (card->config.src_width > 1024 || card->config.src_height > 1024) && (card->card_type != MGA_G550 && card->config.src_height > 1024) ){
+				printk(KERN_ERR "mga_vid: overlay sizes bigger than 1024x1024 are not supported with this card type, rescale the picture in software\n");
+
+
 				return(-EFAULT);
 			}
 			
@@ -729,14 +745,6 @@ static int mga_vid_ioctl(struct inode *inode, struct file *file, unsigned int cm
 #ifdef MP_DEBUG
 			printk(KERN_DEBUG "mga YUV buffer base: 0x%X\n", card->src_base);
 #endif			
-			
-			if (card->is_g400) 
-			  card->config.card_type = MGA_G400;
-			else
-			  card->config.card_type = MGA_G200;
-		       
-			card->config.ram_size = card->ram_size;
-
 			if (copy_to_user((mga_vid_config_t *) arg, &card->config, sizeof(mga_vid_config_t)))
 			{
 				printk(KERN_ERR "mga_vid: failed copy to userspace\n");
@@ -810,7 +818,7 @@ static int mga_vid_ioctl(struct inode *inode, struct file *file, unsigned int cm
 	return 0;
 }
 
-static void cards_init(mga_card_t * card, struct pci_dev * dev, int card_number, int is_g400);
+static void cards_init(mga_card_t * card, struct pci_dev * dev, int card_number, int card_type);
 
 // returns the number of found cards
 static int mga_vid_find_card(void)
@@ -844,22 +852,22 @@ static int mga_vid_find_card(void)
 		case PCI_DEVICE_ID_MATROX_G550:
 			mga_dev_name = "MGA G550";
 			printk(KERN_INFO "mga_vid: Found %s at %s\n", mga_dev_name,  pci_name(dev));
-			cards_init(card, dev, mga_cards_num - 1, 1);
+			cards_init(card, dev, mga_cards_num - 1, MGA_G550);
 			break;
 		case PCI_DEVICE_ID_MATROX_G400:
 			mga_dev_name = "MGA G400/G450";
 			printk(KERN_INFO "mga_vid: Found %s at %s\n", mga_dev_name, pci_name(dev));
-			cards_init(card, dev, mga_cards_num - 1, 1);
+			cards_init(card, dev, mga_cards_num - 1, MGA_G400);
 			break;
 		case PCI_DEVICE_ID_MATROX_G200_AGP:
 			mga_dev_name = "MGA G200 AGP";
 			printk(KERN_INFO "mga_vid: Found %s at %s\n", mga_dev_name, pci_name(dev));
-			cards_init(card, dev, mga_cards_num - 1, 0);
+			cards_init(card, dev, mga_cards_num - 1, MGA_G200);
 			break;
 		case PCI_DEVICE_ID_MATROX_G200_PCI:
 			mga_dev_name = "MGA G200";
 			printk(KERN_INFO "mga_vid: Found %s at %s\n", mga_dev_name, pci_name(dev));
-			cards_init(card, dev, mga_cards_num - 1, 0);
+			cards_init(card, dev, mga_cards_num - 1, MGA_G200);
 			break;
 		default:
 			mga_cards_num--;
@@ -1030,14 +1038,15 @@ static struct file_operations mga_vid_fops =
 	release: 	mga_vid_release
 };
 
-static void cards_init(mga_card_t * card, struct pci_dev * dev, int card_number, int is_g400)
+static void cards_init(mga_card_t * card, struct pci_dev * dev, int card_number, int card_type)
 {
 	unsigned int card_option;
 
 	memset(card,0,sizeof(mga_card_t));
 
 	card->pci_dev = dev;
-	card->is_g400 = is_g400;
+
+	card->card_type = card_type;
 
 	card->param_buff = kmalloc(PARAM_BUFF_SIZE,GFP_KERNEL);
 	if(card->param_buff) card->param_buff_size = PARAM_BUFF_SIZE;
@@ -1066,7 +1075,7 @@ static void cards_init(mga_card_t * card, struct pci_dev * dev, int card_number,
 
 #else
 
-	    if (card->is_g400){
+	    if (card->card_type != MGA_G200){
 		switch((card_option>>10)&0x17){
 		    // SDRAM:
 		    case 0x00:
